﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace AutoUpdater
{
    public partial class Form1 : Form
    {
        string AutoUpdaterPath { get; set; }
        string OptionsFilePath { get; set; }
        Thread LoopThread { get; set; }

        string PlayersInstruction = "ex: \"foot,suomi\"";
        string TimeInstruction = "ex: \"1 hour and 30 minutes\", \"23 hours\", \"1 day\"";

        public Form1(string[] args)
        {
            InitializeComponent();

            urlbox.Text = PlayersInstruction;
            AutoUpdaterPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) + "\\CML\\AutoUpdater";
            OptionsFilePath = AutoUpdaterPath + "\\options.txt";

            if (File.Exists(OptionsFilePath))
            {
                foreach (string Line in File.ReadAllLines(OptionsFilePath))
                {
                    try
                    {
                        string Value = Line.Substring(Line.IndexOf('=') + 1);
                        switch (Line.Substring(0, Line.IndexOf('=')))
                        {
                            case "updateosrs":
                                updateCrystalMathLabsToolStripMenuItem.Checked = (Value[0] == '1');
                                break;
                            case "updaters3":
                                updateCMLrs3ToolStripMenuItem.Checked = (Value[0] == '1');
                                break;
                            case "updatedm":
                                updateCMLdeadmanToolStripMenuItem.Checked = (Value[0] == '1');
                                break;
                            case "names":
                                urlbox.Text = Value;
                                break;
                            case "time":
                                secbox.Text = Value;
                                break;
                        }
                    }
                    catch { }
                }
                UpdateStatus(null, "Loaded " + OptionsFilePath);
                urlbox_Leave(null, null);
                secbox_Leave(null, null);
            }
            UpdateStatus(null, "Written by Foot from CrystalMathLabs.com");
            UpdateStatus(null, "Send feedback here: http://crystalmathlabs.com/tracker/contact.php");
            UpdateStatus(null, "Update multiple players by separating names with commas.");

            foreach (string s in args)
            {
                switch (s)
                {
                    case "-autorun":
                        if (LoopThread == null)
                        {
                            StartUpdating();
                        }
                        break;
                }
            }
        }

        private void startbut_Click(object sender, EventArgs e)
        {
            if (LoopThread == null)
            {
                StartUpdating();
            }
            else
            {
                StopUpdating();
            }
        }

        private void StartUpdating()
        {
            if (urlbox.Text == PlayersInstruction)
            {
                UpdateStatus(null, "You must enter a list of players first.");
                return;
            }
            if (secbox.Text == TimeInstruction)
            {
                UpdateStatus(null, "You must enter a time period first.");
                return;
            }
            try
            {
                if (short_string_to_time(secbox.Text) > 3599)
                {
                    this.Text = "Automatic Tracker Updater (Started)";
                    startbut.Text = "Stop";

                    UpdateStatus(null, "Started updating.");

                    LoopThread = new Thread(Loop);
                    LoopThread.Start();
                }
                else
                {
                    UpdateStatus(null, "Set a time value of more than 3599 seconds.");
                }
            }
            catch (FormatException)
            {
                UpdateStatus(null, "Time value is not in correct format.");
            }
        }

        void StopUpdating()
        {
            LoopThread.Abort();
            LoopThread = null;
            this.Text = "Automatic Tracker Updater";
            startbut.Text = "Start";
            UpdateStatus(null, "Stopped updating.");
        }

        string getPlayerName(string name)
        {
            string ret = "";
            for (int i = 0; i < name.Length; i++)
            {
                char c = name[i];
                if (isAlphaNumeric((byte)c))
                {
                    ret += name[i];
                }
                else
                {
                    ret += ' ';
                }
            }
            ret = ret.ToLower().Trim();
            if (ret.Length > 12)
            {
                return ret.Substring(0, 12);
            }
            else if (ret.Length == 0)
            {
                return null;
            }
            else
            {
                return ret;
            }
        }

        bool isAlphaNumeric(byte b)
        {
            return (47 < b && b < 58) ||
                    (64 < b && b < 91) ||
                    (96 < b && b < 123) || b == 32;
        }

        void UpdatePlayer(string Player, string Tracker = "tracker", bool Condition = true)
        {
            if (Condition)
            {
                try
                {
                    int code = HTTPQuery("http://crystalmathlabs.com/" + Tracker + "/api.php?type=update&player=" + Player);
                    UpdateStatus(Player, "Updated " + (Tracker == "tracker" ? "tracker-osrs" : Tracker));
                }
                catch (Exception e)
                {
                    UpdateStatus(Player, "Error connecting to the Crystal Math Labs server: \n\t" + e.Message);
                }
                Thread.Sleep(2000);
            }
        }

        void Loop()
        {
            string[] Players = null;
            int SleepSeconds = 3600;
            try
            {
                this.Invoke((MethodInvoker)delegate
                {
                    Players = urlbox.Text.Split(',');
                    SleepSeconds = short_string_to_time(secbox.Text);
                });
            }
            catch { }

            int i = 0;
            for (; i < Players.Length; i++)
            {
                Players[i] = getPlayerName(Players[i]);
            }

            i = 1;
            while (true)
            {
                UpdateStatus(null, "Loop #" + i + " started.");

                foreach (string Player in Players)
                {
                    if (Player != null)
                    {
                        UpdateStatus(Player, "Updating...");
                        UpdatePlayer(Player, "tracker", updateCrystalMathLabsToolStripMenuItem.Checked);
                        UpdatePlayer(Player, "tracker-rs3", updateCMLrs3ToolStripMenuItem.Checked);
                        UpdatePlayer(Player, "tracker-dm", updateCMLdeadmanToolStripMenuItem.Checked);
                    }
                }

                UpdateStatus(null, "Loop #" + i + " completed. Waiting " + SleepSeconds + " seconds.");
                i++;
                Thread.Sleep(SleepSeconds * 1000);
            }
        }

        int HTTPQuery(string addr)
        {
            HttpWebResponse wr = (HttpWebResponse)WebRequest.Create(addr).GetResponse();
            int code = (int)wr.StatusCode;
            wr.Close();
            return code;
        }
         
        int short_string_to_time(string str)
        {
            str = str.ToLower();
            char[] time_symbols = new char[] { 'y', 'M', 'w', 'd', 'h', 'm', 's' };
            int[] time_units = new int[] { 31536000, 2678400, 604800, 86400, 3600, 60, 1 };

            int ret = 0;
            string CurrentNumber = "";
            for (int i = 0; i < str.Length; i++)
            {
                int c = (int)str[i];
                if (47 < c && c < 58)
                {
                    CurrentNumber += str[i];
                }
                else
                {
                    for (int j = 0; j < 7; j++)
                    {
                        if (CurrentNumber != "" && str[i] == time_symbols[j])
                        {
                            ret += time_units[j] * Int32.Parse(CurrentNumber);
                            CurrentNumber = "";
                            break;
                        }
                    }
                }
            }

            if (CurrentNumber != "")
            {
                ret += Int32.Parse(CurrentNumber);
            }

            return ret;
        }

        void UpdateStatus(string prompt, string message)
        {
            if (Thread.CurrentThread == LoopThread)
            {
                try
                {
                    this.Invoke((MethodInvoker)delegate
                    {
                        _UpdateStatus(prompt, message);
                    });
                }
                catch { }
            }
            else
            {
                _UpdateStatus(prompt, message);
            }
        }

        void _UpdateStatus(string prompt, string message)
        {
            updatebox.Text += (DateTime.Now.ToString("d/M/yyyy HH:mm:ss") +
                (prompt == null ? "" : " (" + prompt + ")") + ": " + message + Environment.NewLine);
            updatebox.SelectionStart = updatebox.TextLength;
            updatebox.ScrollToCaret();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                string[] Lines = new string[]
                {
                    "updateosrs=" + (updateCrystalMathLabsToolStripMenuItem.Checked ? 1 : 0),
                    "updaters3=" + (updateCMLrs3ToolStripMenuItem.Checked ? 1 : 0),
                    "updatedm=" + (updateCMLdeadmanToolStripMenuItem.Checked ? 1 : 0),
                    "names=" + urlbox.Text,
                    "time=" + secbox.Text
                };
                if (!Directory.Exists(AutoUpdaterPath))
                {
                    Directory.CreateDirectory(AutoUpdaterPath);
                }
                File.WriteAllLines(OptionsFilePath, Lines);
            }
            catch { }

            sysTray.Visible = false;
            if (LoopThread != null)
            {
                LoopThread.Abort();
                LoopThread = null;
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == this.WindowState)
            {
                this.Hide();
                sysTray.Visible = true;
            }
            else if (FormWindowState.Normal == this.WindowState)
            {
                sysTray.Visible = false;
                this.Show();
            }
        }

        private void sysTray_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            sysTray.Visible = false;
            this.Show();
            this.WindowState = FormWindowState.Normal;
        }

        private void visitCrystalMathLabsTrackerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://crystalmathlabs.com/tracker/");
        }

        private void visitRS3TrackerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://crystalmathlabs.com/tracker-rs3/");
        }

        private void visitDeadmanTrackerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://crystalmathlabs.com/tracker-dm/");
        }

        private void urlbox_Leave(object sender, EventArgs e)
        {
            if (urlbox.Text == "")
            {
                urlbox.Text = PlayersInstruction;
            }
        }

        private void urlbox_Enter(object sender, EventArgs e)
        {
            if (urlbox.Text == PlayersInstruction)
            {
                urlbox.Text = "";
            }
        }

        private void secbox_Leave(object sender, EventArgs e)
        {
            if (secbox.Text == "")
            {
                secbox.Text = TimeInstruction;
            }
        }

        private void secbox_Enter(object sender, EventArgs e)
        {
            if (secbox.Text == TimeInstruction)
            {
                secbox.Text = "";
            }
        }
    }
}
